"use strict";

var _require = require('react-dom'),
    render = _require.render;

var _require2 = require('react'),
    createElement = _require2.createElement;
/* eslint-disable no-param-reassign, no-use-before-define */


var app;
var root;
var handleModelError;
var currentRouter = null;
var mountedModels = new WeakSet();

var hmrRender = function hmrRender(router) {
  app.router(router);
  render(createElement(app.start()), root);
};

var hot = {
  patch: function patch(inst, container) {
    if (app) {
      console.error("[hot.patch] You have been patch app, don't patch app twice");
      return inst;
    }

    if (!isDvaInstance(inst)) {
      console.error('[hot.patch] app should be a `dva` instance');
      return inst;
    }

    app = inst;

    if (container !== undefined) {
      hot.setContainer(container);
    }

    patchAppStart(inst);
    patchAppModel(inst);
    patchAppRouter(inst);
    console.log('[dva-hot] enabled.');
    return inst;
  },
  setContainer: function setContainer(container) {
    container = normalizeContainer(container);

    if (!container) {
      console.error('container should be selector or Element');
    } else {
      root = container;
    }

    return container;
  },
  onModelError: function onModelError(fn) {
    handleModelError = fn;
  },
  model: function model(sourceModule, handleError) {
    if (!isNodeModule(sourceModule)) {
      throw Error('[hot.model] should pass `module` first');
    }

    if (sourceModule.hot) {
      return function (model) {
        if (!isDvaModel(model)) {
          throw Error('[hot.model] model shoule be a obj and has `namespace` prop');
        }

        sourceModule.hot.accept(handleError || handleModelError);
        sourceModule.hot.dispose(function (data) {
          if (mountedModels.has(model)) {
            mountedModels.delete(model);
            model = undefined;
            data.enabled = true;
          }
        });

        if (shouldReplaceModule(sourceModule)) {
          if (app.replaceModel) {
            app.replaceModel(model);
          } else {
            app.unmodel(model.namespace);
            app.model(model);
          }
        }

        return model;
      };
    }

    return passthrough;
  },
  router: function router(sourceModule, handleError) {
    if (!isNodeModule(sourceModule)) {
      throw Error('[hot.router] should pass `module` first');
    }

    if (sourceModule.hot) {
      return function (router) {
        if (!isFunction(router)) {
          throw Error('[hot.router] router should be a function');
        }

        sourceModule.hot.accept(handleError);
        sourceModule.hot.dispose(function (data) {
          data.enabled = currentRouter === router;
          router = undefined;
        });

        if (shouldReplaceModule(sourceModule)) {
          replaceRouter(router);
        }

        return router;
      };
    }

    return passthrough;
  }
};
module.exports = hot;

function isHTMLElement(node) {
  return typeof node === 'object' && node !== null && node.nodeType && node.nodeName;
}

function isString(str) {
  return typeof str === 'string';
}

function isFunction(func) {
  return typeof func === 'function';
}

function isDvaInstance(inst) {
  return inst && isFunction(inst.start) && isFunction(inst.model) && isFunction(inst.router) && isFunction(inst.use);
}

function isDvaModel(obj) {
  return obj && obj.namespace;
}

function isObject(obj) {
  return obj && typeof obj === 'object';
}

function passthrough(x) {
  return x;
}

function isNodeModule(x) {
  return isObject(x) && 'id' in x;
}

function replaceRouter(router) {
  currentRouter = router;
  hmrRender(router);
}

function normalizeContainer(container) {
  if (isString(container)) {
    container = document.querySelector(container);
  }

  if (isHTMLElement(container)) {
    return container;
  }

  return null;
}

function shouldReplaceModule(sourceModule) {
  return app && root && sourceModule.hot.data && sourceModule.hot.data.enabled;
}

function patchAppStart(inst) {
  var oldStart = inst.start;

  inst.start = function (container) {
    if (container !== undefined) {
      container = hot.setContainer(container);
    }

    if (container) {
      app.use({
        onHmr: function onHmr(r) {
          hmrRender = r;
        }
      });
    }

    var rootElement = oldStart.call(inst, container);
    patchAppModel(inst);
    return rootElement;
  };
}

function patchAppModel(inst) {
  var oldMethod = inst.model;

  inst.model = function (m) {
    if (isDvaModel(m)) {
      mountedModels.add(m);
    }

    return oldMethod.call(inst, m);
  };
}

function patchAppRouter(inst) {
  var oldMethod = inst.router;

  inst.router = function (r) {
    if (isFunction(r)) {
      currentRouter = r;
    }

    return oldMethod.call(inst, r);
  };
}